/*
 * ssidsniff - a tool for wireless network reconnaissance
 * Copyright (C) 2002-2007 Kostas Evangelinos <kos@bastard.net>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/*
 * $Id: xpaudio-bsd.c,v 1.4 2007-06-20 01:47:12 kos Exp $
 *
 * This is too easy. All hail sun.
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>


#ifdef HAVE_SYS_AUDIOIO_H
#include <sys/types.h>
#include <sys/ioctl.h>
#include <string.h>
#include <sys/audioio.h>
#endif

#ifdef HAVE_MACHINE_PCAUDIOIO_H
#include <sys/types.h>
#include <sys/ioctl.h>
#include <string.h>
#include <machine/pcaudioio.h>
#endif

#include "xmalloc.h"
#include "xpaudio.h"

#define XPA_DEFAULTDEVICE "/dev/audio"

xpaudio_t *
xpaudio_new(char *ud) {
  xpaudio_t *xpa;
  
  xpa = xcalloc(1, sizeof(xpaudio_t));
  if(ud)
    strncpy(xpa->device, ud, XPA_LINELEN);
  else
    strncpy(xpa->device, XPA_DEFAULTDEVICE, XPA_LINELEN);
  
  if((xpa->fd = open(xpa->device, O_WRONLY|O_NDELAY)) < 0) {
    free(xpa);
    return (xpaudio_t *)NULL;
  }

  return xpa;
}

int
xpaudio_destroy(xpaudio_t *xpa) {
  close(xpa->fd);
  xfree(xpa);
  return 0;
}

int
xpaudio_set(xpaudio_t *xpa, unsigned int param, int val) {
  audio_info_t ai;

  if(ioctl(xpa->fd, AUDIO_GETINFO, &ai))
    return -1;
#ifdef HAVE_SYS_AUDIOIO_H
  ai.mode = AUMODE_PLAY;
#endif

  switch(param) {
  case XPA_RATE:
    ai.play.sample_rate = val;
    xpa->rate = val;
    break;
  case XPA_NCHANNELS:
    ai.play.channels = val;
    xpa->channels = val;
    break;
  case XPA_ENCODING: /* XXX: Unsupported */
    break;
  default:
    return -3;
  }
  
  if(ioctl(xpa->fd, AUDIO_SETINFO, &ai))
    return -2;

  return 0;
}

int 
xpaudio_get(xpaudio_t *xpa, unsigned int param) {
  
  switch(param) {
  case XPA_RATE:
    return(xpa->rate);
    break;
  case XPA_NCHANNELS:
    return(xpa->channels);
    break;
  case XPA_ENCODING:
    return(xpa->encoding);
    break;
  default:    
    return 1;
  }  
  return 0;
}

char *
xpaudio_describe(xpaudio_t *xpa) {
  return "SUN/BSD sound module";
}

int
xpaudio_send(xpaudio_t *xpa, char *data, unsigned int len) {

  write(xpa->fd, data, len);
  //ioctl(xpa->fd, AUDIO_DRAIN, &foo);
  return 0;
}
