/*
 * ssidsniff - a tool for wireless network reconnaissance
 * Copyright (C) 2002-2007 Kostas Evangelinos <kos@bastard.net>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/*
 * $Id: xpaudio-esd.c,v 1.3 2007-06-20 01:47:12 kos Exp $
 *
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>


#ifdef HAVE_ESD_H
#include <esd.h>
#endif

#include "xmalloc.h"
#include "xpaudio.h"

extern char *esdserver;

xpaudio_t *
xpaudio_new(char *ud) {
  xpaudio_t *xpa;
  int esd;
  
  xpa = xcalloc(1, sizeof(xpaudio_t));
  
  if((esd = esd_open_sound(NULL)) < 0) {
    free(xpa);
    return (xpaudio_t *)NULL;
  }  

  xpa->fd = esd_play_stream(ESD_PLAY|ESD_STREAM|ESD_BITS16|ESD_MONO, XPA_RATE_DEFAULT, NULL, PACKAGE);
  return xpa;
}

int
xpaudio_destroy(xpaudio_t *xpa) {
  close(xpa->fd);
  xfree(xpa);
  return 0;
}

int
xpaudio_set(xpaudio_t *xpa, unsigned int param, int val) {
  esd_server_info_t info;

  info.rate = xpa->rate;
  info.format = ESD_PLAY|ESD_STREAM;

  switch(param) {
  case XPA_RATE:
    info.rate = val;
    xpa->rate = val;
    break;
  case XPA_NCHANNELS:
    info.format &= (~ESD_MASK_CHAN);
    if(val == 1)
      info.format |= ESD_MONO;
    if(val == 2)
      info.format |= ESD_STEREO;
    xpa->channels = val;
    break;
  case XPA_ENCODING:
    info.format &= (~ESD_MASK_BITS);
    if(val |= XPA_ENCODING_8BIT)
      info.format |= ESD_BITS8;
    if(val |= XPA_ENCODING_16BIT)
      info.format |= ESD_BITS16;
    xpa->encoding = val;
    break;
  default:
    return -1;
  }

  /* duh */
  close(xpa->fd);
  xpa->fd = esd_play_stream(ESD_PLAY|ESD_STREAM|info.format, info.rate, NULL, PACKAGE);
  return 0;
}

int 
xpaudio_get(xpaudio_t *xpa, unsigned int param) {
  return -1;
}

char *
xpaudio_describe(xpaudio_t *xpa) {
  return "EsounD sound module";
}

int
xpaudio_send(xpaudio_t *xpa, char *data, unsigned int len) {

  write(xpa->fd, data, len);
  esd_audio_flush();
  return 0;
}
