/*
 * ssidsniff - a tool for wireless network reconnaissance
 * Copyright (C) 2002-2007 Kostas Evangelinos <kos@bastard.net>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/*
 * $Id: xpaudio-oss.c,v 1.6 2007-06-20 01:47:12 kos Exp $
 *
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>


#ifdef HAVE_LINUX_SOUNDCARD_H
#include <linux/soundcard.h>
#endif

#include "xmalloc.h"
#include "xpaudio.h"

#define XPA_DEFAULTDEVICE "/dev/dsp"

xpaudio_t *
xpaudio_new(char *ud) {
  xpaudio_t *xpa;
  int frag;
  
  xpa = xcalloc(1, sizeof(xpaudio_t));
  if(ud)
    strncpy(xpa->device, ud, XPA_LINELEN);
  else
    strncpy(xpa->device, XPA_DEFAULTDEVICE, XPA_LINELEN);
  
  if((xpa->fd = open(xpa->device, O_WRONLY|O_NDELAY)) < 0) {
    free(xpa);
    return (xpaudio_t *)NULL;
  }

  frag = 2 << 16;
  frag += (XPA_FRAGSIZE);
  xpa->blksz = XPA_FRAGSIZE;
  xpaudio_set(xpa, XPA_RATE, 44100);
  xpaudio_set(xpa, XPA_NCHANNELS, 1);
  ioctl(xpa->fd, SNDCTL_DSP_SETFRAGMENT, &frag);
  ioctl(xpa->fd, SNDCTL_DSP_GETBLKSIZE, &xpa->blksz);
/*   ioctl(xpa->fd, SNDCTL_DSP_NONBLOCK); */
/*   ioctl(xpa->fd, SNDCTL_DSP_SYNC); */
/*   printf("blocksize %d\n", xpa->blksz); */
/*   sleep(2); */
  return xpa;
}

int
xpaudio_destroy(xpaudio_t *xpa) {
  close(xpa->fd);
  xfree(xpa);
  return 0;
}

int
xpaudio_set(xpaudio_t *xpa, unsigned int param, int val) {
  int ret;

  switch(param) {
  case XPA_RATE:
    ret = ioctl(xpa->fd, SNDCTL_DSP_SPEED, &val);
    xpa->rate = val;
    break;
  case XPA_NCHANNELS:
    ret = ioctl(xpa->fd, SNDCTL_DSP_CHANNELS, &val);
    xpa->channels = val;
    break;
  case XPA_ENCODING:
    ret = ioctl(xpa->fd, SNDCTL_DSP_SETFMT, &val);
    xpa->encoding = val;
    break;
  default:
    return -1;
  }
  
  return ret;
}

int 
xpaudio_get(xpaudio_t *xpa, unsigned int param) {
  
  switch(param) {
  case XPA_RATE:
    return(xpa->rate);
    break;
  case XPA_NCHANNELS:
    return(xpa->channels);
    break;
  case XPA_ENCODING:
    return(xpa->encoding);
    break;
  default:    
    return 1;
  }  
  return 0;
}

char *
xpaudio_describe(xpaudio_t *xpa) {
  return "OSS sound module";
}

int
xpaudio_send(xpaudio_t *xpa, char *data, unsigned int len) {
  int foo;
  char buffer[XPA_FRAGSIZE];

  memset(buffer, '\0', XPA_FRAGSIZE);
  memcpy(buffer, data, MIN(len, XPA_FRAGSIZE));
  write(xpa->fd, buffer, XPA_FRAGSIZE);
  ioctl(xpa->fd, SNDCTL_DSP_SYNC, &foo);
  return 0;
}
